#include "utils.h"
#include <QString>

static char fbnum = '0';
static char overlaynum = '1';
static QString overlay_blank("/sys/class/graphics/fb1/blank");
static QString fb_blank("/sys/class/graphics/fb0/blank");
static QString overlay_devname("/dev/fb1");
static QString fb_devname("/dev/fb0");
static char const * const videodevs[] = {
	" device=/dev/video17"
,	" device=/dev/video19"
,	" device=/dev/video21"
};

static QString videodev("");
static QString res_spec ;

/**
 * Search the program configuration for the value of a specified field
 * @param key	key to query value for
 * @param buffer	character buffer to which to copy value found
 * @return	true if operation successful, false if not
 */
bool
get_config(std::string key, char buffer[])
{
	std::ifstream config;
	std::string value;
	bool key_found = false;

	config.open(CONFIG_NAME);
	if (!config.is_open())
		return false;

	while (config.good() && !key_found) {
		size_t delimiter;
		size_t key_start;
		std::string line;

		getline (config, line);
		if (!config.good())
			break;

		// Check if key exists in line
		key_start = line.find(key);
		if (key_start == std::string::npos)
			continue;

		// Check if delimiter exists in line
		delimiter = line.find("=");
		if (delimiter == std::string::npos)
			continue;

		// key must end one character before delimiter for this to be
		// a complete and certain match
		if (delimiter == key.length())
		{
			value = line.substr (delimiter + 1, std::string::npos);
			strcpy(buffer, value.c_str());
			if (0 == strncmp("pipeline",key.c_str(),8)) {
				strcat(buffer,qPrintable(videodev));
				strcat(buffer,qPrintable(res_spec));
			}
			key_found = true;
			break;
		}
	}
	config.close();
	return key_found;
}

// Set fb/overlay blend level
// 0 means no foreground; 255 means fully foreground
// Return true if operation was a success
// Note: call this AFTER gstreamer initializations: something in them resets the alpha
bool
set_alpha(int alpha_arg)
{
	int fb_fd;
	int result;

	struct mxcfb_gbl_alpha alpha;
	alpha.enable = 1;
	alpha.alpha = alpha_arg;

	fb_fd = open(qPrintable(fb_devname), O_RDWR, 0);
	if(fb_fd < 0)
	{
		printf("\nCould not open %s. Now exiting\n",qPrintable(fb_devname));
		exit(-1);
	}

	result = ioctl(fb_fd, MXCFB_SET_GBL_ALPHA, &alpha);
	if(result < 0)
	{
		printf("\nUnable to set alpha for %s\n",qPrintable(fb_devname));
	}

	close(fb_fd);
	return true;
}

// Clears and wakes primary framebuffer
// Returns true if successful
bool
wake()
{
	printf("-----------------%s\n", __func__ );
	std::ofstream fd_fb_blank;
	fd_fb_blank.open(qPrintable(fb_blank));
	fd_fb_blank << 1;
	fd_fb_blank.close();

	fd_fb_blank.open(qPrintable(fb_blank));
	fd_fb_blank << 0;
	fd_fb_blank.close();

	if(fd_fb_blank.fail())
	{
		printf("\nFailed to open fb%c for clearing and waking. Now exiting",fbnum);
		exit(-1);
	}

	std::ofstream fd_overlay_blank;
	fd_overlay_blank.open(qPrintable(overlay_blank));
	fd_overlay_blank << 1;
	fd_overlay_blank.close();

	fd_overlay_blank.open(qPrintable(overlay_blank));
	fd_overlay_blank << 0;
	fd_overlay_blank.close();

	if(fd_overlay_blank.fail())
	{
		printf("\nFailed to open %s for clearing and waking. Now exiting",
		       qPrintable(overlay_blank));
	}

	return true;
}

// Resizes fb1 overlay to the size of fb
// Returns 'true' if successful
bool
resize_overlay()
{
	struct fb_var_screeninfo fb_screeninfo;
	struct fb_var_screeninfo overlay_screeninfo;

	int fb_fd;
	int overlay_fd;

	int overlay_new_xres;	// New resolutions for overlay resolution
	int overlay_new_yres;	// Based on fb resolutions

	// Open fb to read screen dimensions
	printf("\nOpening %s for reading",qPrintable(fb_devname));
	fb_fd = open(qPrintable(fb_devname), O_RDONLY);
	if(fb_fd < 0)
	{
		printf("\nFailed to open %s. Now exiting",qPrintable(fb_devname));
		exit(-1);
	}

	// Open overlay to read and set screen dimensions
	printf("\nOpening %s for reading and writing", qPrintable(overlay_devname));
	overlay_fd = open(qPrintable(overlay_devname), O_RDWR);
	if(overlay_fd < 0)
	{
		printf("\nFailed to open %s. Now exiting", qPrintable(overlay_devname));
		exit(-1);
	}

	// Read fb screen dimensions
	printf("\nReading %s variable screen information", qPrintable(fb_devname));
	if(ioctl(fb_fd, FBIOGET_VSCREENINFO, &fb_screeninfo) < 0)
	{
		printf("\nFailed to obtain %s variable screen info. Now exiting",qPrintable(fb_devname));
		exit(-1);
	}

	printf("\nfb xres: %d", fb_screeninfo.xres);
	printf("\nfb yres: %d", fb_screeninfo.yres);

	// Set overlay new resolutions
	overlay_new_xres = fb_screeninfo.xres;
	overlay_new_yres = fb_screeninfo.yres;

	printf("\nClosing fb file descriptor");
	close(fb_fd);

	// Obtain overlay variable screen info
	printf("\nObtaining %s variable screen information",qPrintable(overlay_devname));
	if(ioctl(overlay_fd, FBIOGET_VSCREENINFO, &overlay_screeninfo) < 0)
	{
		printf("\nFailed to obtain %s variable screen info. Now exiting", qPrintable(overlay_devname));
		exit(-1);
	}

	// Set overlay variable screen info
	printf("\nSetting %s resolutions to match fb", qPrintable(overlay_devname));
	overlay_screeninfo.xres = overlay_new_xres;
	overlay_screeninfo.yres = overlay_new_yres;

	// Write new resolution values to overlay
	printf("\nAttempting to \'put\' new values of xres and yres to %s\n", qPrintable(overlay_devname));
	if(ioctl(overlay_fd, FBIOPUT_VSCREENINFO, &overlay_screeninfo) < 0)
	{
		printf("\nError writing new resolution values to %s. Now exiting", qPrintable(overlay_devname));
		exit(-1);
	}

	// Close overlay file descriptor, clean up
	printf("\nClosing overlay file descriptor");
	printf("\n");
	close(overlay_fd);

	res_spec = QString(" disp-width=%1 disp-height=%2").arg(fb_screeninfo.xres).arg(fb_screeninfo.yres);
	printf("resolution spec: %s (len %u)\n", qPrintable(res_spec),res_spec.size());

	return true;
}

void init_display()
{
	char *fbdev = getenv("QWS_DISPLAY");
	if (fbdev) {
		char *end=fbdev+strlen(fbdev)-1;
                fbnum = *end ;
	        overlaynum = fbnum+1;
		overlay_blank = QString("/sys/class/graphics/fb%1/blank").arg(overlaynum);
		fb_blank = QString("/sys/class/graphics/fb%1/blank").arg(fbnum);
		overlay_devname = QString("/dev/fb%1").arg(overlaynum);
		fb_devname = QString("/dev/fb%1").arg(fbnum);
		printf("devnames are %s and %s\n"
		       "overlays %s and %s\n",
		       qPrintable(overlay_devname), qPrintable(fb_devname),
		       qPrintable(overlay_blank), qPrintable(fb_blank));
       }
       printf("fbnum == %c, idx %d\n", fbnum, fbnum-'0');
       printf("videodev(%s)\n", videodevs[(fbnum-'0')/2]);
       videodev = QString(videodevs[(fbnum-'0')/2]);
       printf("videodev: %s\n", qPrintable(videodev));
}
